@@ -4,8 +4,11 @@ use strict;
 use URI;
 use Carp ();
 
-use vars qw($VERSION);
-$VERSION = "5.817";
+use vars qw($VERSION $Encode_available);
+$VERSION = "5.827";
+
+eval { require Encode };
+$Encode_available = !$@;
 
 my %form_tags = map {$_ => 1} qw(input textarea button select option);
 
@@ -56,45 +59,67 @@ The following methods are available:
 
 =over 4
 
-=item @forms = HTML::Form->parse( $response )
+=item @forms = HTML::Form->parse( $html_document, $base_uri )
 
-=item @forms = HTML::Form->parse( $html_document, $base )
+=item @forms = HTML::Form->parse( $html_document, base => $base_uri, %opt )
 
-=item @forms = HTML::Form->parse( $html_document, %opt )
+=item @forms = HTML::Form->parse( $response, %opt )
 
 The parse() class method will parse an HTML document and build up
 C<HTML::Form> objects for each <form> element found.  If called in scalar
 context only returns the first <form>.  Returns an empty list if there
 are no forms to be found.
 
-The $base is the URI used to retrieve the $html_document.  It is
-needed to resolve relative action URIs.  If the document was retrieved
-with LWP then this this parameter is obtained from the
-$response->base() method, as shown by the following example:
+The required arguments is the HTML document to parse ($html_document) and the
+URI used to retrieve the document ($base_uri).  The base URI is needed to resolve
+relative action URIs.  The provided HTML document should be a Unicode string
+(or US-ASCII).
+
+By default HTML::Form assumes that the original document was UTF-8 encoded and
+thus encode forms that don't specify an explict I<accept-charset> as UTF-8.
+The charset assumed can be overridden by providing the C<charset> option to
+parse().  It's a good idea to be explict about this parameter as well, thus
+the recommended simplest invocation becomes:
+
+    my @forms = HTML::Form->parse(
+        Encode::decode($encoding, $html_document_bytes),
+        base => $base_uri,
+	charset => $encoding,
+    );
+
+If the document was retrieved with LWP then the response object provide methods
+to obtain a proper value for C<base> and C<charset>:
 
     my $ua = LWP::UserAgent->new;
     my $response = $ua->get("http://www.example.com/form.html");
     my @forms = HTML::Form->parse($response->decoded_content,
-				  $response->base);
+	base => $response->base,
+	charset => $response->content_charset,
+    );
 
-The parse() method can parse from an C<HTTP::Response> object
+In fact, the parse() method can parse from an C<HTTP::Response> object
 directly, so the example above can be more conveniently written as:
 
     my $ua = LWP::UserAgent->new;
     my $response = $ua->get("http://www.example.com/form.html");
     my @forms = HTML::Form->parse($response);
 
-Note that any object that implements a decoded_content() and base() method
-with similar behaviour as C<HTTP::Response> will do.
+Note that any object that implements a decoded_content(), base() and
+content_charset() method with similar behaviour as C<HTTP::Response> will do.
 
-Finally options might be passed in to control how the parse method
-behaves.  The following options are currently recognized:
+Additional options might be passed in to control how the parse method
+behaves.  The following are all the options currently recognized:
 
 =over
 
 =item C<< base => $uri >>
 
-Another way to provide the base URI.
+This is the URI used to retrive the original document.  This option is not optional ;-)
+
+=item C<< charset => $str >>
+
+Specify what charset the original document was encoded in.  This is used as
+the default for accept_charset.  If not provided this defaults to "UTF-8".
 
 =item C<< verbose => $bool >>
 
@@ -121,6 +146,7 @@ sub parse
     die "Failed to create HTML::TokeParser object" unless $p;
 
     my $base_uri = delete $opt{base};
+    my $charset = delete $opt{charset};
     my $strict = delete $opt{strict};
     my $verbose = delete $opt{verbose};
 
@@ -136,6 +162,14 @@ sub parse
 	    Carp::croak("HTML::Form::parse: No \$base_uri provided");
 	}
     }
+    unless (defined $charset) {
+	if (ref($html) and $html->can("content_charset")) {
+	    $charset = $html->content_charset;
+	}
+	unless ($charset) {
+	    $charset = "UTF-8";
+	}
+    }
 
     my @forms;
     my $f;  # current form
@@ -151,6 +185,8 @@ sub parse
 	    $f = $class->new($attr->{'method'},
 			     $action,
 			     $attr->{'enctype'});
+            $f->accept_charset($attr->{'accept-charset'}) if $attr->{'accept-charset'};
+	    $f->{default_charset} = $charset;
 	    $f->{attr} = $attr;
 	    $f->strict(1) if $strict;
             %openselect = ();
@@ -271,6 +307,8 @@ sub new {
     $self->{method} = uc(shift  || "GET");
     $self->{action} = shift  || Carp::croak("No action defined");
     $self->{enctype} = lc(shift || "application/x-www-form-urlencoded");
+    $self->{accept_charset} = "UNKNOWN";
+    $self->{default_charset} = "UTF-8";
     $self->{inputs} = [@_];
     $self;
 }
@@ -289,7 +327,11 @@ sub push_input
     my @extra;
     push(@extra, readonly => 1) if $type eq "hidden";
     push(@extra, strict => 1) if $self->{strict};
-
+    if ($type eq "file" && exists $attr->{value}) {
+	# it's not safe to trust the value set by the server
+	# the user always need to explictly set the names of files to upload
+	$attr->{orig_value} = delete $attr->{value};
+    }
     delete $attr->{type}; # don't confuse the type argument
     my $input = $class->new(type => $type, %$attr, @extra);
     $input->add_to_form($self);
@@ -317,11 +359,22 @@ I<method> to.
 This method gets/sets the encoding type for the form data.  It is a
 string like "application/x-www-form-urlencoded" or "multipart/form-data".
 
+=item $accept = $form->accept_charset
+
+=item $form->accept_charset( $new_accept )
+
+This method gets/sets the list of charset encodings that the server processing
+the form accepts. Current implementation supports only one-element lists.
+Default value is "UNKNOWN" which we interpret as a request to use document
+charset as specified by the 'charset' parameter of the parse() method. To
+encode character strings you should have modern perl with Encode module. On
+older perls the setting of this attribute has no effect.
+
 =cut
 
 BEGIN {
     # Set up some accesor
-    for (qw(method action enctype)) {
+    for (qw(method action enctype accept_charset)) {
 	my $m = $_;
 	no strict 'refs';
 	*{$m} = sub {
@@ -641,6 +694,13 @@ sub make_request
     my $enctype = $self->{'enctype'};
     my @form    = $self->form;
 
+    my $charset = $self->accept_charset eq "UNKNOWN" ? $self->{default_charset} : $self->accept_charset;
+    if ($Encode_available) {
+        foreach my $fi (@form) {
+            $fi = Encode::encode($charset, $fi) unless ref($fi);
+        }
+    }
+
     if ($method eq "GET") {
 	require HTTP::Request;
 	$uri = URI->new($uri, "http");
@@ -1322,6 +1382,10 @@ If the input is of type C<file>, then it has these additional methods:
 This is just an alias for the value() method.  It sets the filename to
 read data from.
 
+For security reasons this field will never be initialized from the parsing
+of a form.  This prevents the server from triggering stealth uploads of
+arbitrary files from the client machine.
+
 =cut
 
 sub file {
